if (MODID == 18 and MODVERSION < 52)
	-- I tried to be polite about it, so now I'll just be annoying about it
	hud.add(function(v)
		v.drawFill(0, 0, 320, 200, 31);
		v.draw(
			0, 0,
			v.cachePatch("GMA_PRCY")
		);

		local piracyLines = {
			"E-102 Gamma is not compatible",
			"with SRB2 versions that are",
			"older than v2.2.11.",
			"",
			"Please update your copy of SRB2",
			"before playing as Gamma.",
		};

		local spacing = 20;
		local y = 100 - (#piracyLines * (spacing / 2));
		for i = 1,#piracyLines do
			v.drawString(160, y, piracyLines[i], V_ALLOWLOWERCASE, "center");
			y = $1 + spacing;
		end
	end);

	addHook("ThinkFrame", function()
		if (server and server.valid)
			for player in players.iterate do
				player.powers[pw_nocontrol] = UINT16_MAX;
				player.pflags = $1 | PF_GODMODE | PF_STASIS | PF_JUMPSTASIS;

				if (player.mo and player.mo.valid)
					player.mo.flags = $1 & ~MF_SHOOTABLE;
				end
			end

			S_ChangeMusic("GMAOVR", false);
		end
	end);

	error("Please update SRB2! Gamma is not compatible with versions older than v2.2.11.");
	return;
end

if (Gamma != nil)
	-- Multiple Gammas loaded?
	error("Gamma was already loaded.");
	return;
end

// DEFINE GLOBAL TABLE
rawset(_G, "Gamma", {});

// Library / dependencies
dofile("lib/customhud.lua");
dofile("lib/vectorlib.lua");

// Important definitions
dofile("definitions.lua");

// Init hook system
dofile("hooks.lua");

// Object hooking
dofile("mobj/edits-base.lua"); // Vanilla object edits
dofile("mobj/edits-thz.lua");
dofile("mobj/edits-dsz.lua");
dofile("mobj/edits-cez.lua");
dofile("mobj/edits-acz.lua");
dofile("mobj/edits-hhz.lua");
dofile("mobj/edits-fhz.lua");
dofile("mobj/edits-atz.lua");
dofile("mobj/edits-stalagmite.lua");
dofile("mobj/edits-disco.lua");
dofile("mobj/edits-xmas.lua");
dofile("mobj/edits-mario.lua");
dofile("mobj/custom.lua"); // Custom object examples

// Abilty functions
dofile("ability/shoot.lua");
dofile("ability/blaster.lua");
dofile("ability/aim.lua");
dofile("ability/jet.lua");
dofile("ability/drive.lua");
dofile("ability/coop.lua");

// Animation systems
dofile("anim/partdefs.lua");
dofile("anim/animationdefs.lua");
dofile("anim/animator.lua");
dofile("anim/ghosts.lua");

// Special EFX functions
dofile("efx/shot.lua");
dofile("efx/laser.lua");
dofile("efx/target.lua");
dofile("efx/jet.lua");
dofile("efx/coop.lua");
dofile("efx/blaster.lua");
dofile("efx/dust.lua");
dofile("efx/sfx.lua");

// Timer system functions
dofile("time.lua");

// Extra Smart Tether support
dofile("tether.lua");

// Extra Battle support
dofile("battle.lua");

/* still WIP for now
if (srb2p)
	// Extra Persona support
	dofile("persona.lua");
end
*/

// Main gameplay loop
dofile("main.lua");

// Main HUD loop
dofile("hud.lua");
