-- ========
-- Gamma custom map objects

-- I had a few people tell me they wanted to make Gamma map sets, so why not sweeten the deal a bit?
-- Also serves as an example for how Gamma's hooks can work for your own objects.

-- Simply copy this Lua file, and all of the sprites in "sprites/objs",
-- into your map project to start using these objects.
-- ========

if (GammaObjectsDefined != nil)
	-- Only add them once!
	return;
end
rawset(_G, "GammaObjectsDefined", true);

--
-- Wall bomb
--

freeslot(
	"MT_GAMMA_BOMB",
	"S_GAMMA_BOMB",
	"S_GAMMA_BOMB_EXPLODE",
	"S_GAMMA_BOMB_OVERLAY",
	"S_GAMMA_BOMB_OVERLAY2",
	"S_GAMMA_BOMB_OVERLAY3",
	"S_GAMMA_BOMB_OVERLAY4",
	"S_GAMMA_BOMB_OVERLAY5",
	"S_GAMMA_BOMB_OVERLAY6",
	"S_GAMMA_BOMB_JUNK",
	"S_GAMMA_BOMB_JUNK2",
	"S_GAMMA_BOMB_JUNK3",
	"S_GAMMA_BOMB_JUNK4",
	"SPR_GMBM"
);

mobjinfo[MT_GAMMA_BOMB] = {
	--$Name Bustable Wall Dynamite
	--$Sprite GMBMA0
	--$Category E-102 Gamma
	--$AngleText Tag (Binary)
	doomednum = 1789,
	spawnstate = S_GAMMA_BOMB,
	deathstate = S_GAMMA_BOMB_EXPLODE,
	deathsound = sfx_cybdth,
	spawnhealth = 1,
	radius = 24*FRACUNIT,
	height = 64*FRACUNIT,
	flags = MF_NOGRAVITY|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOCLIPTHING|MF_SCENERY
}

states[S_GAMMA_BOMB] = {SPR_GMBM, A, -1, nil, 0, 0, S_NULL}
states[S_GAMMA_BOMB_EXPLODE] = {SPR_NULL, A, 16, nil, 0, 0, S_NULL}

states[S_GAMMA_BOMB_OVERLAY] = {SPR_GMBM, B|FF_SEMIBRIGHT, 35, nil, 0, 0, S_GAMMA_BOMB_OVERLAY2}
states[S_GAMMA_BOMB_OVERLAY2] = {SPR_GMBM, C|FF_FULLBRIGHT, 1, nil, 0, 0, S_GAMMA_BOMB_OVERLAY3}
states[S_GAMMA_BOMB_OVERLAY3] = {SPR_GMBM, D|FF_FULLBRIGHT, 1, nil, 0, 0, S_GAMMA_BOMB_OVERLAY4}
states[S_GAMMA_BOMB_OVERLAY4] = {SPR_GMBM, E|FF_FULLBRIGHT, 1, nil, 0, 0, S_GAMMA_BOMB_OVERLAY5}
states[S_GAMMA_BOMB_OVERLAY5] = {SPR_GMBM, F|FF_FULLBRIGHT, 1, nil, 0, 0, S_GAMMA_BOMB_OVERLAY6}
states[S_GAMMA_BOMB_OVERLAY6] = {SPR_GMBM, G|FF_FULLBRIGHT, 1, nil, 0, 0, S_GAMMA_BOMB_OVERLAY}

states[S_GAMMA_BOMB_JUNK] = {SPR_GMBM, H, 35, nil, 0, 0, S_NULL}
states[S_GAMMA_BOMB_JUNK2] = {SPR_GMBM, I, 35, nil, 0, 0, S_NULL}
states[S_GAMMA_BOMB_JUNK3] = {SPR_GMBM, J, 35, nil, 0, 0, S_NULL}
states[S_GAMMA_BOMB_JUNK4] = {SPR_GMBM, K, 35, nil, 0, 0, S_NULL}

local BombList = {};

addHook("MapChange", function()
	BombList = {};
end);

addHook("NetVars", function(net)
	BombList = net(BombList);
end)

addHook("MobjSpawn", function(mo)
	local overlay = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_OVERLAY);

	if (overlay and overlay.valid)
		mo.target = overlay;
		overlay.target = mo;
		overlay.state = S_GAMMA_BOMB_OVERLAY;
		overlay.tics = P_RandomRange(1, $1);
	end
end, MT_GAMMA_BOMB);

local function RemoveBombOverlay(mo)
	if (mo.target and mo.target.valid)
		P_RemoveMobj(mo.target);
	end
end

addHook("MobjRemoved", RemoveBombOverlay, MT_GAMMA_BOMB);

addHook("MobjDeath", function(mo, inflictor, source)
	RemoveBombOverlay(mo);

	if (source and source.valid)
		mo.angle = R_PointToAngle2(mo.x, mo.y, source.x, source.y);
	else
		mo.angle = P_RandomRange(0, 359) * ANG1;
	end

	for i = 0,3 do
		local junk = P_SpawnMobjFromMobj(
			mo,
			0, 0, 0,
			MT_BOSSJUNK
		);

		junk.state = (S_GAMMA_BOMB_JUNK + i);

		if (i >= 2)
			junk.angle = mo.angle + ANGLE_90;
		else
			junk.angle = mo.angle - ANGLE_90;
		end

		P_InstaThrust(junk, junk.angle, 4*junk.scale);

		if (i & 1)
			P_SetObjectMomZ(junk, 2*FRACUNIT, false);
			P_InstaThrust(junk, junk.angle, 8*junk.scale);
		else
			P_SetObjectMomZ(junk, 4*FRACUNIT, false);
			P_InstaThrust(junk, junk.angle, 4*junk.scale);
		end
	end
end, MT_GAMMA_BOMB);

addHook("MobjThinker", function(mo)
	if (mo.health == 0) and (leveltime % 2 == 0)
		A_BossScream(mo, 0, 0);
	end
end, MT_GAMMA_BOMB);

local function AddBombToList(mo, tag)
	if (BombList[tag] == nil)
		-- Create new bomb list
		BombList[tag] = {
			bombs = {},
			rovers = {},
			init = false, -- Rovers are not ready by this point, so just tell the game to init later.
		};
	end

	-- Simply add to whatever's already there :)
	table.insert(BombList[tag].bombs, mo);
end

addHook("MapThingSpawn", function(mo, mt)
	if not (udmf)
		-- Non-UDMF map behavior
		local tag = mt.angle;
		AddBombToList(mo, tag);
	else
		for tag in mt.taglist.iterate() do
			AddBombToList(mo, tag);
		end
	end
end, MT_GAMMA_BOMB);

local function InitBombData(bombData, tag)
	local noRovers = true;

	for sector in sectors.iterate do
		for rover in sector.ffloors()
			local roverTags = rover.master.taglist;

			if (taglist.find(roverTags, tag) == true)
				-- Store all rovers that match this tag.
				table.insert(bombData.rovers, rover);
				noRovers = false;
			end
		end
	end

	-- Invalid bomb setup.
	if (noRovers == true)
		print("\x82WARNING:\x80 No FOFs found for Gamma bomb tagged "..tag);
		return false;
	end

	return true;
end

local function IterateList(list, func)
	for i = 1,#list do
		func(list);
	end
end

local AddedGammaHooks = false;
local function CheckForGammaHooks()
	if (AddedGammaHooks == true)
		-- Already added our Gamma-related hooks.
		return;
	end

	if (Gamma == nil)
		-- Gamma isn't added yet...
		return;
	end

	-- Add our Gamma-related hooks!
	Gamma.SetTargetFunc(MT_GAMMA_BOMB, function(mo, aimList, owner)
		return true;
	end);

	Gamma.SetDamageFunc(MT_GAMMA_BOMB, function(target, shot, source)
		P_KillMobj(target, shot, source);
		return Gamma.SHOT_EXPLODE;
	end);

	Gamma.SetPointsFunc(MT_GAMMA_BOMB, function(mo)
		-- In SA1, these don't add to combo.
		-- But in SA2, they do!
		return 1;
	end);

	-- Don't do this more than once.
	AddedGammaHooks = true;
end

addHook("ThinkFrame", function()
	CheckForGammaHooks();

	if (leveltime <= 2)
		-- defrosting time
		return;
	end

	local removeTags = {};

	for tag,bombData in pairs(BombList)
		if (bombData == nil)
			continue;
		end

		if (bombData.init == false)
			bombData.init = true;
			local success = InitBombData(bombData, tag);

			if (success == false)
				table.insert(removeTags, tag);
			end

			continue;
		end

		local roversExist = false;
		for _,rover in pairs(bombData.rovers)
			if not (rover and rover.valid)
				continue;
			end

			if (rover.flags & FF_EXISTS)
				roversExist = true;
				break;
			end
		end

		if (roversExist == false)
			-- Our FOFs no longer exist. They were likely busted via other means.
			-- So destroy all of the bombs.

			for _,bomb in pairs(bombData.bombs)
				if (bomb and bomb.valid and bomb.health > 0)
					P_KillMobj(bomb);
				end
			end

			table.insert(removeTags, tag);
			continue;
		end

		local bombsExist = false;
		for _,bomb in pairs(bombData.bombs)
			if (bomb and bomb.valid and bomb.health > 0)
				bombsExist = true;
				break;
			end
		end

		if (bombsExist == false)
			-- All bombs destroyed, so destroy all of our rovers too!
			for _,rover in pairs(bombData.rovers)
				if not (rover.flags & FF_EXISTS)
					continue;
				end

				EV_CrumbleChain(nil, rover);

				if (rover.master.flags & ML_EFFECT5)
					local execute = P_AproxDistance(rover.master.dx, rover.master.dy) >> FRACBITS;
					P_LinedefExecute(execute);
				end
			end

			table.insert(removeTags, tag);
			continue;
		end
	end

	for _,tag in pairs(removeTags)
		-- Only remove tags after we're done iterating.
		if (tag == nil)
			continue;
		end

		--print("Removed tag "..tag);
		BombList[tag] = nil;
	end
end);

--
-- Extra Time monitor
--

freeslot(
	"MT_EXTRATIME_BOX",
	"S_EXTRATIME_BOX",
	"MT_EXTRATIME_ICON",
	"S_EXTRATIME_ICON1",
	"S_EXTRATIME_ICON2",
	"SPR_TVET"
);

function A_GammaExtraTime(actor, var1, var2)
	if (var1 <= 0)
		-- Invalid seconds.
		return;
	end

	if not (actor.target and actor.target.valid)
		-- Powerup has no target.
		return;
	end

	local player = actor.target.player;
	if not (player and player.valid)
		-- Powerup has no (valid) target.
		return;
	end

	if (Gamma != nil)
		Gamma.GiveArbitraryTime(var1, player);
	end
end

mobjinfo[MT_EXTRATIME_BOX] = {
	--$Name Extra Time
	--$Sprite TVETA0
	--$Category E-102 Gamma
	doomednum = 1790,
	spawnstate = S_EXTRATIME_BOX,
	painstate = S_EXTRATIME_BOX,
	deathstate = S_BOX_POP1,
	deathsound = sfx_pop,
	spawnhealth = 1,
	damage = MT_EXTRATIME_ICON,
	radius = 18*FRACUNIT,
	height = 40*FRACUNIT,
	flags = MF_SOLID|MF_SHOOTABLE|MF_MONITOR
}

states[S_EXTRATIME_BOX] = {SPR_TVET, A, 2, nil, 0, 0, S_BOX_FLICKER}

mobjinfo[MT_EXTRATIME_ICON] = {
	spawnstate = S_EXTRATIME_ICON1,
	seesound = sfx_pop,
	spawnhealth = 1,
	speed = 2*FRACUNIT,
	damage = 62*FRACUNIT,
	radius = 8*FRACUNIT,
	height = 14*FRACUNIT,
	flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_SCENERY|MF_NOGRAVITY|MF_BOXICON
}

states[S_EXTRATIME_ICON1] = {SPR_TVET, C|FF_ANIMATE, 18, nil, 3, 4, S_EXTRATIME_ICON2}
states[S_EXTRATIME_ICON2] = {SPR_TVET, C, 18, A_GammaExtraTime, 30, 0, S_NULL}

--[[
freeslot(
	"MT_EXTRATIME_GOLDBOX",
	"S_EXTRATIME_GOLDBOX"
);

mobjinfo[MT_EXTRATIME_GOLDBOX] = {
	--$Name Extra Time (Respawn)
	--$Sprite TVETB0
	--$Category E-102 Gamma
	doomednum = ????,
	spawnstate = S_EXTRATIME_GOLDBOX,
	painstate = S_EXTRATIME_GOLDBOX,
	deathstate = S_GOLDBOX_OFF1,
	attacksound = sfx_monton,
	deathsound = sfx_pop,
	spawnhealth = 1,
	damage = MT_EXTRATIME_ICON,
	radius = 20*FRACUNIT,
	height = 44*FRACUNIT,
	flags = MF_SOLID|MF_SHOOTABLE|MF_MONITOR|MF_GRENADEBOUNCE
}

states[S_EXTRATIME_GOLDBOX] = {SPR_TVET, B, 2, A_GoldMonitorSparkle, 0, 0, S_GOLDBOX_FLICKER}
--]]
